#!/usr/bin/env python3
"""Arrrgs root command handler"""
from arrrgs import arg, command, run


@command(
    arg("--rage", "-r", action='store_true', help="Rage mod"),
    root=True
)
def hello(args, name: str):
    """Prints hello message to current user"""
    message = f"Hello, {name}"
    if args.rage:
        message = message.upper()
    print(message)

@command()
def bye(_, name: str):
    """Prints bye message to current user"""
    print(f"Bye, {name}")

async def prepare(args):
    """Creates app context"""
    return args, "Mikhael"

if __name__ == "__main__":
    run(prepare=prepare)



"""Arrrgs argument parser"""
from argparse import ArgumentParser

parser = ArgumentParser()
parser.set_defaults(
    root_command_handler=None,
    root_command_args=None,
)
command_subparsers = parser.add_subparsers(dest="cmd")



"""Arrrgs starter"""
import asyncio
from argparse import Namespace
from inspect import signature
from logging import DEBUG
from typing import Any, Callable, Dict, List, Optional, Tuple, Union

from .arguments import add_args
from .log import get_logger
from .parser import parser

log = get_logger("arrrgs.run")

ContextPreparer = Callable[[Namespace, Any], Tuple[Namespace, Any]]

def run(
    ctx: Any = None,
    prepare: Optional[ContextPreparer] = None,
    after: Optional[Callable] = None,
    debug=False
):
    """Runs application"""
    asyncio.run(async_run(ctx, prepare, after, debug))

async def async_run(
    ctx: Any = None,
    prepare: Union[ContextPreparer, None] = None,
    after: Optional[Callable] = None,
    debug = False,
):
    """Runs application asynchronously"""
    args, argv = parser.parse_known_args()
    if debug:
        log.setLevel(DEBUG)
        log.info("Running in debug mode")
    if argv:
        parser.error(f"unrecognized arguments: {' '.join(argv)}")
    handler: Callable
    if prepare is not None:
        log.info("Running 'prepare' callback")
        (prepared_args, prepared_context) = await _run_safe(prepare, args, ctx)
        # Updating values
        args = prepared_args
        ctx = prepared_context
    if args.cmd is None:
        if args.root_command_handler is None:
            log.info("root command handler is not found. Printing help")
            parser.print_help()
            return
        add_args(parser, args.root_command_args)
        args = parser.parse_args()
        handler = args.root_command_handler
    else:
        handler = args.func
    log.info(args)
    await _run_safe(handler, args, ctx)
    if after is not None:
        await _run_safe(after, args, ctx)

def _prepare_args(handler: Callable, parsed_args: Namespace, ctx: Any) -> List[str]:
    sig = signature(handler)
    params_count = len(sig.parameters)
    args = []
    if params_count >= 1:
        args.append(parsed_args)
    if params_count >= 2:
        args.append(ctx)
    return args

async def _run_safe(handler: Callable, parsed_args: Dict[str, Any], ctx: Any):
    args = _prepare_args(handler, parsed_args, ctx)
    return await _run_async_safe(handler, args)

async def _run_async_safe(handler: Callable, args: List[any]):
    if asyncio.iscoroutinefunction(handler):
        return await handler(*args)
    return handler(*args)



#!/usr/bin/env python3
"""Basic Arrrgs example"""
from arrrgs import command, run


@command(aliases=["l", "list"])
def list_numbers():
    """Prints list of numbers"""
    print("1, 2, 3")

if __name__ == "__main__":
    run()



"""Arrrgs command"""
from argparse import ArgumentParser
from typing import Callable, List, Tuple

from .arguments import add_args
from .parser import command_subparsers, parser


def register_handler(
    parent: ArgumentParser,
    func: Callable,
    args: Tuple[str, ...],
    name:str=None
):
    """Add new command handler to parser"""
    if name is not None:
        command_name = name
    else:
        command_name = func.__name__
    cmd_parser = parent.add_parser(command_name, description=func.__doc__)
    add_args(cmd_parser, args)
    cmd_parser.set_defaults(func=func)

def command(
        *args,
        name: str = None,
        root = False,
        parent = command_subparsers,
        root_parser = parser,
        aliases: List[str] = None
    ):
    """Decorator to define a new command"""
    def decorator(func: Callable):
        if root:
            root_parser.set_defaults(
                root_command_handler=func,
                root_command_args=list(args),
            )
        register_handler(parent, func, args, name)
        if aliases is not None:
            for alias in aliases:
                register_handler(parent, func, args, alias)
    return decorator



#!/usr/bin/env python3
"""Basic Arrrgs example"""
from os import getlogin

from arrrgs import command, run


@command()
def hello():
    """Prints hello message to current user"""
    print(f"Hello, {getlogin()}")

if __name__ == "__main__":
    run()



#!/usr/bin/env python3
"""Arrrgs arguments example"""
from arrrgs import command, run


class User:
    """Represents user"""

    def say_hello(self):
        print(f"Hello, {self._name}!")

    def say_bye(self):
        print(f"Bye, {self._name}!")

    def __init__(self, name):
        self._name = name

    def get_name(self):
        """Returns user name"""
        return self._name

@command()
def hello(_, user: User):
    """Prints hello message to current user"""
    user.say_hello()

async def say_bye(_, user: User):
    """Prints bye message to current user"""
    user.say_bye()

async def prepare(args):
    """Creates app context"""
    return args, User("Mikhael")

if __name__ == "__main__":
    run(
        prepare=prepare,
        after=say_bye
    )
